{% load wagtailcore_tags wagtailimages_tags %}
<div class="row">
{% for image in value.images %}
    <div class="col-md-{{ column_width }}">
        {% image image fill-400x400 class="img-responsive img-thumbnail" %}
    </div>
{% endfor %}
</div>
