.carousel {
    .item-inner {
        height: 300px;
        background-repeat: no-repeat;
        background-size: cover;
        background-position: center;
        display: flex;
    }

    .carousel-caption {
        bottom: 0;
        right: 0;
        left: 0;
        padding-bottom: 40px;
        background: rgba(0, 0, 0, 0.7);
    }

    &.with-thumbnails {
        margin-bottom: 115px;
    }

    .carousel-thumbnails {
        bottom: -100px;
        margin-left: 0;
        margin-right: 0;
        padding: 15px;
        background: #000;

        li {
            width: auto;
            height: auto;
            text-indent: initial;
            opacity: 0.5;

            &.active {
                opacity: 1;
            }
        }
    }
}
