# Generated by Django 2.0.3 on 2018-04-13 11:32

from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.blocks
import wagtail.core.fields
import wagtail_extensions.blocks
import wagtailgeowidget.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0040_page_draft_title'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactDetailsTestSetting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('locations', wagtail.core.fields.StreamField((('location', wagtail.core.blocks.StructBlock((('name', wagtail.core.blocks.CharBlock()), ('address', wagtail.core.blocks.StructBlock((('lines', wagtail.core.blocks.ListBlock(wagtail.core.blocks.CharBlock(label='Line', required=False))),), required=False)), ('point', wagtailgeowidget.blocks.GeoBlock(required=False)), ('departments', wagtail.core.blocks.ListBlock(wagtail.core.blocks.StructBlock((('name', wagtail.core.blocks.CharBlock(required=False)), ('phones', wagtail_extensions.blocks.StrippedListBlock(wagtail_extensions.blocks.PhoneBlock(required=False))), ('email', wagtail.core.blocks.EmailBlock(required=False)), ('primary', wagtail.core.blocks.BooleanBlock(default=False, required=False))), label='department', required=False))), ('opening_times', wagtail.core.blocks.StructBlock((('times', wagtail.core.blocks.ListBlock(wagtail.core.blocks.StructBlock((('weekday', wagtail.core.blocks.ChoiceBlock(choices=[(0, 'Monday'), (1, 'Tuesday'), (2, 'Wednesday'), (3, 'Thursday'), (4, 'Friday'), (5, 'Saturday'), (6, 'Sunday'), (7, 'Public holiday')], required=False)), ('label', wagtail.core.blocks.CharBlock(help_text='Optionally override default weekday label', required=False)), ('date', wagtail.core.blocks.DateBlock(help_text='Optionally specify a day these times are for', required=False)), ('start', wagtail.core.blocks.TimeBlock(required=False)), ('end', wagtail.core.blocks.TimeBlock(required=False)), ('closed', wagtail.core.blocks.BooleanBlock(default=False, required=False))), required=False))),), required=False)), ('primary', wagtail.core.blocks.BooleanBlock(default=False, required=False))))),))),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
