{% load static %}

<script src="{% static "wagtail_extensions/js/maps.js" %}"></script>
<script async defer src="https://maps.googleapis.com/maps/api/js?key={{ api_key }}&amp;callback=init_maps"></script>
